"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addDownloadCLI = void 0;
const commander_1 = require("commander");
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const myst_cli_utils_1 = require("myst-cli-utils");
const doi_utils_1 = __importDefault(require("doi-utils"));
const chalk_1 = __importDefault(require("chalk"));
const session_1 = require("../session");
const types_1 = require("../types");
const jats_1 = require("../jats");
const myst_common_1 = require("myst-common");
const unist_util_select_1 = require("unist-util-select");
const download_1 = require("../download");
const resolvers_1 = require("../resolvers");
const utils_1 = require("../utils");
function hasValidExtension(output) {
    return ['.xml', '.jats'].includes((0, path_1.extname)(output).toLowerCase());
}
function downloadJats(session, urlOrDoi, output) {
    return __awaiter(this, void 0, void 0, function* () {
        if (fs_1.default.existsSync(urlOrDoi)) {
            throw new Error(`File "${urlOrDoi}" is local and cannot be downloaded!`);
        }
        if (!(doi_utils_1.default.validate(urlOrDoi) || (0, myst_cli_utils_1.isUrl)(urlOrDoi))) {
            throw new Error(`Path must be a URL or DOI, not "${urlOrDoi}"`);
        }
        if (!hasValidExtension(output)) {
            session.log.warn(`The extension ${(0, path_1.extname)(output)} is not a valid extension for JATS, try using ".xml" or ".jats"`);
        }
        const data = yield (0, download_1.downloadJatsFromUrl)(session, urlOrDoi, resolvers_1.DEFAULT_RESOLVERS);
        (0, myst_cli_utils_1.writeFileToFolder)(output, data);
        return data;
    });
}
function parseJats(session, file) {
    return __awaiter(this, void 0, void 0, function* () {
        const toc = (0, myst_cli_utils_1.tic)();
        if (fs_1.default.existsSync(file)) {
            session.log.debug(`Found ${file} locally, parsing`);
            const data = fs_1.default.readFileSync(file).toString();
            session.log.debug(toc(`Parsed JATS file from disk in %s`));
            return new jats_1.Jats(data);
        }
        if (doi_utils_1.default.validate(file) || (0, myst_cli_utils_1.isUrl)(file)) {
            const data = yield (0, download_1.downloadJatsFromUrl)(session, file, resolvers_1.DEFAULT_RESOLVERS);
            session.log.debug(toc(`Downloaded and parsed JATS file in %s`));
            return new jats_1.Jats(data);
        }
        throw new Error(`Could not find ${file} locally, and it doesn't look like a URL or DOI`);
    });
}
function formatLongString(data, offset = 0, length = 88 - offset) {
    const out = [data.slice(0, length)];
    let left = data.slice(length);
    while (left.length > length) {
        out.push(left.slice(0, length).trim());
        left = left.slice(length);
    }
    if (left)
        out.push(left.trim());
    return out.join(`\n${' '.repeat(offset)}`);
}
function formatDictionary(dict, opts) {
    const maxLabel = Object.keys(dict).reduce((a, b) => Math.max(a, b.length), 0);
    return Object.entries(dict)
        .map(([k, t]) => {
        var _a;
        let value = t;
        let color = chalk_1.default.yellow.bold;
        if (t && typeof t === 'object') {
            if (!t.value)
                return null;
            color = (_a = t.label) !== null && _a !== void 0 ? _a : color;
            value = t.value;
        }
        const wrapped = (opts === null || opts === void 0 ? void 0 : opts.wrap) === false
            ? String(value)
            : formatLongString(String(value), maxLabel + 2, opts === null || opts === void 0 ? void 0 : opts.wrap);
        return `${color(k)}:${' '.repeat(maxLabel - k.length + 1)}${wrapped}`;
    })
        .filter((o) => !!o)
        .join('\n');
}
function jatsSummaryCLI(session, file) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const jats = yield parseJats(session, file);
        const summary = {
            DOI: jats.doi ? doi_utils_1.default.buildUrl(jats.doi) : null,
            Title: (_a = (0, myst_common_1.toText)(jats.articleTitle)) === null || _a === void 0 ? void 0 : _a.replace(/\n/g, ' '),
            Date: (0, utils_1.formatDate)((0, utils_1.toDate)(jats.publicationDate)),
            Authors: jats.articleAuthors
                .map((a) => `${(0, myst_common_1.toText)((0, unist_util_select_1.select)(types_1.Tags.givenNames, a))} ${(0, myst_common_1.toText)((0, unist_util_select_1.select)(types_1.Tags.surname, a))}`)
                .join(', '),
            Abstract: (_b = (0, myst_common_1.toText)(jats.abstract)) === null || _b === void 0 ? void 0 : _b.replace(/\n/g, ' '),
            Keywords: jats.keywords.map((k) => (0, myst_common_1.toText)(k)).join(', '),
            License: jats.license['xlink:href'],
            Figures: { label: chalk_1.default.blue.bold, value: String((0, unist_util_select_1.selectAll)(types_1.Tags.fig, jats.body).length) },
            Equations: {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.dispFormula, jats.body).length),
            },
            Tables: {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.table, jats.body).length),
            },
            Code: {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.code, jats.body).length),
            },
            Sections: {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.sec, jats.body).length),
            },
            Paragraphs: {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.p, jats.body).length),
            },
            Citations: { label: chalk_1.default.blue.bold, value: String(jats.references.length) },
            'Cross-References': {
                label: chalk_1.default.blue.bold,
                value: String((0, unist_util_select_1.selectAll)(types_1.Tags.xref, jats.body).length),
            },
            'Sub Articles': { label: chalk_1.default.blue.bold, value: String(jats.subArticles.length) },
        };
        session.log.info(formatDictionary(summary));
    });
}
function jatsReferencesCLI(session, file) {
    return __awaiter(this, void 0, void 0, function* () {
        const jats = yield parseJats(session, file);
        const sorted = jats.references
            .map((ref) => {
            const doi = (0, utils_1.findDoi)(ref);
            const title = (0, myst_common_1.toText)((0, unist_util_select_1.select)(types_1.Tags.articleTitle, ref));
            const year = (0, myst_common_1.toText)((0, unist_util_select_1.select)(types_1.Tags.year, ref));
            const surnames = (0, unist_util_select_1.selectAll)(types_1.Tags.surname, ref);
            const short = surnames.length > 2
                ? (0, myst_common_1.toText)(surnames[0]) + ' et al.'
                : surnames.length === 2
                    ? (0, myst_common_1.toText)(surnames[0]) + ' and ' + (0, myst_common_1.toText)(surnames[1])
                    : (0, myst_common_1.toText)(surnames[0]);
            const s = (0, unist_util_select_1.selectAll)(`[rid=${ref.id}]`, jats.body);
            return {
                Citation: `${short} (${year})`,
                Title: title,
                DOI: doi ? doi_utils_1.default.buildUrl(doi) : null,
                Count: s.length,
            };
        })
            .sort((a, b) => b.Count - a.Count);
        sorted.forEach((r) => {
            session.log.info(formatDictionary(r, { wrap: false }) + '\n');
        });
    });
}
function makeSummaryCLI(program) {
    const command = new commander_1.Command('summary')
        .description('Parse a JATS file and provide a summary')
        .argument('<jats>', 'The JATS file or remote URL to be parsed')
        .action((0, myst_cli_utils_1.clirun)(jatsSummaryCLI, { program, getSession: session_1.getSession }));
    return command;
}
function makeReferencesCLI(program) {
    const command = new commander_1.Command('refs')
        .alias('references')
        .description('Parse a JATS file and provide a summary')
        .argument('<jats>', 'The JATS file or remote URL to be parsed')
        .action((0, myst_cli_utils_1.clirun)(jatsReferencesCLI, { program, getSession: session_1.getSession }));
    return command;
}
function makeDownloadCLI(program) {
    const command = new commander_1.Command('download')
        .description('Parse a JATS file and provide a summary')
        .argument('<url>', 'The JATS url or a DOI')
        .argument('<output>', 'The JATS url or a DOI')
        .action((0, myst_cli_utils_1.clirun)(downloadJats, { program, getSession: session_1.getSession }));
    return command;
}
function addDownloadCLI(program) {
    program.addCommand(makeDownloadCLI(program));
    program.addCommand(makeSummaryCLI(program));
    program.addCommand(makeReferencesCLI(program));
}
exports.addDownloadCLI = addDownloadCLI;
